/**
 * PETALS: PETALS Services Platform Copyright (C) 2009 EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 * Initial developer(s): EBM WebSourcing
 */
package org.ow2.petals.messaging.framework.event;

import java.util.Iterator;
import java.util.LinkedHashSet;

import org.ow2.petals.messaging.framework.task.SyncTaskExecutor;
import org.ow2.petals.messaging.framework.task.TaskExecutor;


/**
 * @author chamerling - eBM WebSourcing
 * 
 */
public class DefaultListenerManager extends AbstractListenerManager {

    private final TaskExecutor taskExecutor = new SyncTaskExecutor();

    /**
     * 
     */
    public DefaultListenerManager() {
        super();
    }

    /**
     * {@inheritDoc}
     */
    public void publishEvent(final Event event) {
        if (event == null) {
            this.log.warn("Event is null");
            return;
        }

        LinkedHashSet<Listener> listener = this.listeners.get(event.getClass());
        if (listener == null) {
            this.log.warn("No listeners found for " + event.getClass());
            return;
        }

        for (Iterator<Listener> it = listener.iterator(); it.hasNext();) {
            final Listener l = it.next();
            this.taskExecutor.execute(new Runnable() {
                public void run() {
                    DefaultListenerManager.this.log.debug("Executing task for event "
                            + event.getClass());
                    l.onEvent(event);
                }
            });
        }
    }
}
